#include "tuya_iot_config.h"
#include "tuya_cloud_types.h"
#include "tuya_iot_com_api.h"
#include "tal_thread.h"
#include "tal_log.h"
#include "tal_system.h"
#include "tuya_ws_db.h"
#include  <stdlib.h>

#include "tkl_gpio.h"

#include "u8g2.h"

/***********************************************************
***********************typedef define***********************
***********************************************************/
#define TASK_IIC_PRIORITY           THREAD_PRIO_1
#define TASK_IIC_SIZE               2048

#define SSD1306_SCL_PIN             TUYA_GPIO_NUM_20
#define SSD1306_SDA_PIN             TUYA_GPIO_NUM_22

/***********************************************************
********************function declaration********************
***********************************************************/


/***********************************************************
***********************variable define**********************
***********************************************************/
STATIC THREAD_HANDLE sg_u8g2_handle;
STATIC THREAD_HANDLE ty_app_thread = NULL;
STATIC THREAD_CFG_T sg_task = {
    .priority = TASK_IIC_PRIORITY,
    .stackDepth = TASK_IIC_SIZE,
    .thrdname = "u8g2"
};

STATIC u8g2_t u8g2;

/***********************************************************
***********************function define**********************
***********************************************************/

STATIC uint8_t u8x8_gpio_and_delay(u8x8_t *u8x8, uint8_t msg, uint8_t arg_int, void *arg_ptr)
{
    // TAL_PR_DEBUG("msg: %d arg_int: %d", msg, arg_int);
    TUYA_GPIO_BASE_CFG_T scl_pin_cfg = {
            .mode = TUYA_GPIO_PUSH_PULL,
            .direct = TUYA_GPIO_OUTPUT,
            .level = TUYA_GPIO_LEVEL_LOW
        };
        TUYA_GPIO_BASE_CFG_T sda_pin_cfg = {
            .mode = TUYA_GPIO_PUSH_PULL,
            .direct = TUYA_GPIO_OUTPUT,
            .level = TUYA_GPIO_LEVEL_LOW
        };
            
    switch(msg)
    {
        case U8X8_MSG_GPIO_AND_DELAY_INIT:
            /* only support for software I2C*/ 
            tkl_gpio_init(SSD1306_SCL_PIN, &scl_pin_cfg);
            tkl_gpio_init(SSD1306_SDA_PIN, &sda_pin_cfg);
            break;

        case U8X8_MSG_DELAY_NANO:
            /* not required for SW I2C */
            break;

        case U8X8_MSG_DELAY_10MICRO:
            /* not used at the moment */
            break;

        case U8X8_MSG_DELAY_100NANO:
            /* not used at the moment */
            break;

        case U8X8_MSG_DELAY_MILLI:
            // delay_micro_seconds(arg_int*1000UL);
            break;

        case U8X8_MSG_DELAY_I2C:
            // arg_int is the I2C speed in 100KHz, e.g. 4 = 400 KHz
            // arg_int=1: delay by 5us, arg_int = 4: delay by 1.25us
            for (uint16_t n = 0; n < (arg_int<=2?160:40); n++)
            {
                __asm__ volatile("nop");
            }            
            break;
    
        case U8X8_MSG_GPIO_I2C_CLOCK:
            // arg_int=0: Output low at I2C clock pin
            // arg_int=1: Input dir with pullup high for I2C clock pin
            if ( arg_int == 0 )
            {
                tkl_gpio_write(SSD1306_SCL_PIN, TUYA_GPIO_LEVEL_LOW);
            }
            else
            {
                tkl_gpio_write(SSD1306_SCL_PIN, TUYA_GPIO_LEVEL_HIGH);
            }
            break;

        case U8X8_MSG_GPIO_I2C_DATA:
            // arg_int=0: Output low at I2C data pin
            // arg_int=1: Input dir with pullup high for I2C data pin
            if ( arg_int == 0 )
            {
                tkl_gpio_write(SSD1306_SDA_PIN, TUYA_GPIO_LEVEL_LOW);
            }
            else
            {
                tkl_gpio_write(SSD1306_SDA_PIN, TUYA_GPIO_LEVEL_HIGH);
            }
            break;

        default:
            break;
    }

    return 1;
}

STATIC VOID draw(u8g2_t *u8g2)
{
    u8g2_SetFontMode(u8g2, 1);	// Transparent
    u8g2_SetFontDirection(u8g2, 0);
    u8g2_SetFont(u8g2, u8g2_font_inb24_mf);
    u8g2_DrawStr(u8g2, 0, 30, "U");
    
    u8g2_SetFontDirection(u8g2, 1);
    u8g2_SetFont(u8g2, u8g2_font_inb30_mn);
    u8g2_DrawStr(u8g2, 21,8,"8");
        
    u8g2_SetFontDirection(u8g2, 0);
    u8g2_SetFont(u8g2, u8g2_font_inb24_mf);
    u8g2_DrawStr(u8g2, 51,30,"g");
    u8g2_DrawStr(u8g2, 67,30,"\xb2");
    
    u8g2_DrawHLine(u8g2, 2, 35, 47);
    u8g2_DrawHLine(u8g2, 3, 36, 47);
    u8g2_DrawVLine(u8g2, 45, 32, 12);
    u8g2_DrawVLine(u8g2, 46, 33, 12);
  
    u8g2_SetFont(u8g2, u8g2_font_4x6_tr);
    u8g2_DrawStr(u8g2, 1,54,"github.com/olikraus/u8g2");
}

/**
* @brief u8g2_task
*
* @return none
*/
VOID u8g2_task(VOID* param)
{
    u8g2_Setup_ssd1306_i2c_128x64_noname_f(&u8g2, U8G2_R0, u8x8_byte_sw_i2c, u8x8_gpio_and_delay);   // init u8g2 structure
    u8g2_InitDisplay(&u8g2);            // send init sequence to the display, display is in sleep mode after this,
    u8g2_SetPowerSave(&u8g2, 0);        // wake up display 

    while(1)
    {
        u8g2_FirstPage(&u8g2);
        do
        {
            draw(&u8g2);
        } while (u8g2_NextPage(&u8g2));

        tal_system_sleep(5000);
     }
}

/**
* @brief u8g2_demo
*
* @return none
*/
VOID u8g2_demo(VOID)
{
    OPERATE_RET rt = OPRT_OK;

    TUYA_CALL_ERR_LOG(tal_thread_create_and_start(&sg_u8g2_handle, NULL, NULL, u8g2_task, NULL, &sg_task));
}

/**
 * @brief tuyaos Initialization thread
 *
 * @param[in] arg: Parameters when creating a task
 *
 * @return none
 */
STATIC VOID_T tuya_app_thread(VOID_T *arg)
{
    OPERATE_RET rt = OPRT_OK;

    /* Initialization LWIP first!!! */
#if defined(ENABLE_LWIP) && (ENABLE_LWIP == 1)
    TUYA_LwIP_Init();
#endif

    /* Initialization, because DB initialization takes a long time, 
     * which affects the startup efficiency of some devices, 
     * so special processing is performed during initialization to delay initialization of DB
     */
#if OPERATING_SYSTEM == SYSTEM_LINUX    
    rt= system("mkdir -p ./tuya_db_files/");
    TUYA_CALL_ERR_LOG(tuya_iot_init_params("./tuya_db_files/", NULL));
#else
    TY_INIT_PARAMS_S init_param = {0};
    init_param.init_db = TRUE;
    strcpy(init_param.sys_env, TARGET_PLATFORM);
    TUYA_CALL_ERR_LOG(tuya_iot_init_params(NULL, &init_param));
#endif

    TAL_PR_NOTICE("sdk_info:%s", tuya_iot_get_sdk_info());                        /* print SDK information */
    TAL_PR_NOTICE("name:%s:%s", APP_BIN_NAME, USER_SW_VER);                       /* print the firmware name and version */
    TAL_PR_NOTICE("firmware compiled at %s %s", __DATE__, __TIME__);              /* print firmware compilation time */
    TAL_PR_NOTICE("system reset reason:[%d]", tal_system_get_reset_reason(NULL)); /* print system reboot causes */

    tal_log_set_manage_attr(TAL_LOG_LEVEL_DEBUG);

    u8g2_demo();

    tal_thread_delete(ty_app_thread);
    ty_app_thread = NULL;
}

/**
 * @brief tuyaos entry function, it will call by main
 *
 * @param[in] none: 
 *
 * @return none
 */
VOID_T tuya_app_main(VOID)
{
    THREAD_CFG_T thrd_param = {4096, 4, "tuya_app_main"};
    TAL_PR_DEBUG("++++++++++tuya_app_main+++++++");
    tal_thread_create_and_start(&ty_app_thread, NULL, NULL, tuya_app_thread, NULL, &thrd_param);
}
